@@ -1,18 +1,15 @@
-use std::time::{Duration, Instant};
-use std::{io::Write, path::PathBuf};
-// use std::io::BufWriter;
-use jokolink::mlink::{CMumbleLink, USEFUL_C_MUMBLE_LINK_SIZE};
-use std::io::{Seek, SeekFrom};
-
-const MUMBLE_REFRESH_INTERVAL: Duration = Duration::from_micros(15);
-const GW2_EXIT_CHECK_INTERVAL: Duration = Duration::from_secs(5);
-
 #[cfg(target_os = "windows")]
-fn main() {
+fn main() -> anyhow::Result<()> {
+    use std::time::Instant;
+    use std::{io::Write, path::PathBuf};
+    // use std::io::BufWriter;
+    use jokolink::mlink::{CMumbleLink, USEFUL_C_MUMBLE_LINK_SIZE};
     use jokolink::win::{create_link_shared_mem, get_xid};
     use log::debug;
+    use std::io::{Seek, SeekFrom};
     // get all the cmd line args and initialize logs.
-    let mumble_link_key = jokolink::cli::create_app();
+    let (mumble_link_key, mumble_refresh_interval, gw2_check_interval) =
+        jokolink::cli::create_app();
     let key: &str = &mumble_link_key;
 
     // create shared memory using the mumble link key
@@ -20,7 +17,13 @@ fn main() {
     debug!("created shared memory. pointer: {:?}", link);
 
     // check that we created shared memory successfully or panic. get ptr to shared memory
-    let link_ptr = link.unwrap();
+    let link_ptr = link.map_err(|e| {
+        log::error!(
+            "unabled to create mumble link shared memory due to error: {:?}",
+            &e
+        );
+        e
+    })?;
 
     // create a shared memory file in /dev/shm/mumble_link_key_name so that jokolay can mumble stuff from there.
     let shmpath: PathBuf = ["z:\\", "dev", "shm", key].iter().collect();
@@ -28,7 +31,13 @@ fn main() {
 
     let shm = std::fs::File::create(&shmpath);
     debug!("shm file created. File: {:?}", &shm);
-    let mut shm = shm.unwrap();
+    let mut shm = shm.map_err(|e| {
+        log::error!(
+            "unable to create the shared memory file in /dev/shm due to error: {:?}",
+            &e
+        );
+        e
+    })?;
 
     // variable to hold the xid.
     let mut xid = None;
@@ -43,10 +52,10 @@ fn main() {
         // copy the bytes from mumble link into shared memory file
         CMumbleLink::copy_raw_bytes_into(link_ptr, &mut buffer[..USEFUL_C_MUMBLE_LINK_SIZE]);
         // we sleep for 10 milliseconds to avoid reading mumblelink too many times. we will read it around 100 times per second
-        std::thread::sleep(MUMBLE_REFRESH_INTERVAL);
+        std::thread::sleep(mumble_refresh_interval);
 
         // every 5 seconds
-        if timer.elapsed() > GW2_EXIT_CHECK_INTERVAL {
+        if timer.elapsed() > gw2_check_interval {
             // reset the timer
             timer = Instant::now();
             // check if mumble is initialized
@@ -62,21 +71,34 @@ fn main() {
                     // successfully got xid
                     if let Some(id) = xid {
                         debug!("mumble link is initialized. got xid");
-                        debug!("Mumble Link data: {:?}", unsafe {*link_ptr});
+                        debug!("Mumble Link data: {:?}", unsafe { *link_ptr });
                         buffer[USEFUL_C_MUMBLE_LINK_SIZE..].copy_from_slice(&id.to_ne_bytes());
                         log::debug!("xid of gw2 window: {:?}", xid);
-
                     }
                 }
-
             } else {
                 log::debug!("the MumbleLink is not init yet. ");
             }
         }
 
         // write buffer to the file
-        shm.write(&buffer).unwrap();
+        shm.write(&buffer).map_err(|e| {
+            log::error!(
+                "could not write to shared memory file due to error: {:?}",
+                &e
+            );
+            e
+        })?;
         // seek back so that we will write to file again from start
-        shm.seek(SeekFrom::Start(0)).unwrap();
+        shm.seek(SeekFrom::Start(0)).map_err(|e| {
+            log::error!(
+                "could not seek to start of shared memory file due to error: {:?}",
+                &e
+            );
+            e
+        })?;
     }
 }
+
+#[cfg(target_os = "linux")]
+fn main() {}
